//CCVisu is a tool for co-change visualization.
//This file is part of CCVisu. 

//Copyright (C) 2005  Dirk Beyer

//CCVisu is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public License
//as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.

//CCVisu is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public License
//along with CCVisu; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//Please find the GNU Lesser General Public License in file
//../license_lgpl.txt or http://www.gnu.org/licenses/lgpl.txt

//Dirk Beyer    (firstname.lastname@epfl.ch)
//Ecole Polytechnique Fdrale de Lausanne (EPFL), Switzerland


package ccvisu;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

/*****************************************************************
 * Writer for displaying the layout on the screen device.
 * @version  $Revision: 1.32 $; $Date: 2005/08/22 12:30:56 $
 * @author   Dirk Beyer
 *****************************************************************/
public class WriterDataGraphicsDISP extends WriterDataGraphics {

  private ScreenDisplay display;
  private Set[] xMap;
  private Set[] yMap;

  // Temporarily associated during callback from ScreenDisplay.
  private Graphics area;
  private int insetleft;

  /**
   * Constructor.
   * @param graph       Graph representation, contains the positions of the vertices.
   * @param hideSource  If true, draw only vertices that are not source of an edge.
   *                    In co-change graphs, all change-transaction vertices 
   *                    are source vertices.
   * @param minVert     Diameter of the smallest vertex.
   * @param fontSize    Font size of vertex annotations.
   * @param backColor   Background color.
   * @param blackCircle If true, draw black circle around each vertex.
   * @param anim        If true, the layout is already displayed
   *                    while the minimizer is still improving it,
   *                    and a simple mouse click on the canvas 
   *                    updates the current layout on the screen.
   *                    If false, the layout is displayed only after
   *                    minimization is completed.
   */
  public WriterDataGraphicsDISP(GraphData graph, 
					            boolean hideSource, 
					            float minVert,
					            int fontSize,
					            Color backColor,
					            boolean blackCircle,
					            boolean anim) {
	super(graph, hideSource, minVert, fontSize, backColor, blackCircle);
	this.display = new ScreenDisplay(this, backColor, anim);
  }

  /*****************************************************************
   * Nothing to do here.
   * The contructor initializes the ScreenDisplay (frame and canvas), 
   * and that calls back to the methods below 
   * (writeDISP, writeLAY, toggleVertexNames, getVertexNames).
   *****************************************************************/
  public void write() {
  }

  /**
   * Writes the layout on the screen device (DISP output format).
   * Call-back method, invoked from <code>ScreenDisplay</code>.
   * @param size         Size of the output drawing quare.
   * @param area         The drawing area of the canvas.
   * @param xCanvasSize  Width of the canvas.
   * @param yCanvasSize  Height of the canvas. 
   * @param insetleft    Left inset of the drawing frame.
   */
  public void writeDISP(int size, 
                        Graphics area, 
                        int xCanvasSize, int yCanvasSize,
                        int insetleft) {
    this.area = area;
    this.insetleft = insetleft;

    // Maps for getting the vertices at mouse positions.
	xMap = new Set[xCanvasSize];
	yMap = new Set[yCanvasSize];
	for(int i = 0; i < xMap.length; ++i) {
		  xMap[i] = new TreeSet();
	}
	for(int i = 0; i < yMap.length; ++i) {
		  yMap[i] = new TreeSet();
	}

	area.setFont(new Font("SansSerif", Font.BOLD, fontSize));

	writeGraphicsLayout(size);
  }

  /**
   * Writes a vertex on screen.
   * @param curVertex  The vertex object, to access vertex attributes.
   * @param xPos       x coordinate of the vertex.
   * @param yPos       y coordinate of the vertex.
   * @param zPos       z coordinate of the vertex.
   * @param dia        Diameter of the vertex.
   */
  public void writeVertex(GraphVertex curVertex, 
                          int xPos, int yPos, int zPos, int dia) {
    // Correction for inset.left.
	xPos = xPos + insetleft;

	// Draw the vertex.
	area.setColor(curVertex.color);
	area.fillOval(xPos - dia/2, yPos - dia/2, dia, dia);
	if (blackCircle) {
	  area.setColor(Color.BLACK);
	  area.drawOval(xPos - dia/2, yPos - dia/2, dia, dia);
	}

	if (curVertex.showName) {
	  // Draw annotation.
	  // Use inverted background color for the annotation.
	  area.setColor(  new Color( 0xffffffff - backColor.getRGB() )  );
	  area.drawString(curVertex.name, xPos + dia/2 + 3, yPos + 3);
	}

	// For interactive annotation: Store positions of the vertices in the maps.
	for (int pos = Math.max(xPos - dia/2, 0); 
		 pos <= Math.min(xPos + dia/2, xMap.length - 1); 
		 ++pos) {
	  xMap[pos].add(curVertex.name);
	}
	for (int pos = Math.max(yPos - dia/2, 0); 
		 pos <= Math.min(yPos + dia/2, yMap.length - 1); 
		 ++pos) {
	  yMap[pos].add(curVertex.name);
	}
  }

  /*****************************************************************
   * Writes layout to file using class <code>WriterDataLAY</code>.
   * Call-back method, invoked from within ScreenDisplay.
   * @param fileName     Name of the output file to write the layout to.
   *****************************************************************/
  public void writeLAY(String fileName) {
	try {
	  PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
	  WriterData 
		dataWriter = new WriterDataLAY (graph, out, hideSource);
	  dataWriter.write();
	  out.close();
	  System.err.println("Wrote layout to output file '" + fileName + "'.");
	}
	catch (Exception e) {
	  System.err.println("Exception while writing file '" + fileName + "': ");
	  System.err.println(e);
	}
  }

  /*****************************************************************
   * Toggle the showName flag of the vertices at the given position.
   * Call-back method, invoked from within ScreenDisplay.
   * @param xPos       x coordinate of the vertex.
   * @param yPos       y coordinate of the vertex.
   *****************************************************************/
  public void toggleVertexNames(int xPos, int yPos) {
	Set tmp = new TreeSet(xMap[xPos]);
	tmp.retainAll(yMap[yPos]);
	Iterator it = tmp.iterator();
	while (it.hasNext()) {
	  GraphVertex 
	  curVertex = (GraphVertex) (graph.nameToVertex.get((String) it.next()));
	  curVertex.toggle_showName();
	}
  }

  /*****************************************************************
   * Compute list of names of the vertices at the given position.
   * Call-back method, invoked from within ScreenDisplay.
   * @param xPos       x coordinate of the vertex.
   * @param yPos       y coordinate of the vertex.
   *****************************************************************/
  public String getVertexNames(int xPos, int yPos) {
	if (xPos >= xMap.length  ||  yPos >= yMap.length) {
	  return new String("");
	}
	Set tmp = new TreeSet(xMap[xPos]);
	tmp.retainAll(yMap[yPos]);
	return(tmp.toString());
  }
};
